﻿#include "framework.h"
#include "hubSwitchSample.h"
#include "ftd2xx.h"
#include <string>
#include <regex>

#define MAX_LOADSTRING 100

// グローバル変数:
HINSTANCE hInst;                                // 現在のインターフェイス
WCHAR szTitle[MAX_LOADSTRING];                  // タイトル バーのテキスト
WCHAR szWindowClass[MAX_LOADSTRING];            // メイン ウィンドウ クラス名
FT_HANDLE ftHandle;
char nextChar = 0x0F;                           // 起動時はすべての出力をオンにする
HWND hEdit;                                     // エディットコントロールのハンドル
int counter = 0;                                // カウンター変数

struct ButtonState {
    bool isRed;
};

ButtonState buttonStates[4] = { {true}, {true}, {true}, {true} };

ATOM MyRegisterClass(HINSTANCE hInstance);
BOOL InitInstance(HINSTANCE, int);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
INT_PTR CALLBACK About(HWND, UINT, WPARAM, LPARAM);
void SetButtonColors(HWND hButton);
BOOL GetSerialNumber(WCHAR* serialNumber, DWORD size);
BOOL IsValidInput(const std::wstring& input);
BOOL WriteToFTDI(const std::wstring& input);

int APIENTRY wWinMain(_In_ HINSTANCE hInstance,
    _In_opt_ HINSTANCE hPrevInstance,
    _In_ LPWSTR    lpCmdLine,
    _In_ int       nCmdShow)
{
    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(lpCmdLine);

    LoadStringW(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
    LoadStringW(hInstance, IDC_HUBSWITCHSAMPLE, szWindowClass, MAX_LOADSTRING);
    MyRegisterClass(hInstance);

    if (!InitInstance(hInstance, nCmdShow))
    {
        return FALSE;
    }

    HACCEL hAccelTable = LoadAccelerators(hInstance, MAKEINTRESOURCE(IDC_HUBSWITCHSAMPLE));

    MSG msg;

    while (GetMessage(&msg, nullptr, 0, 0))
    {
        if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg))
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }

    return (int)msg.wParam;
}

ATOM MyRegisterClass(HINSTANCE hInstance)
{
    WNDCLASSEXW wcex;

    wcex.cbSize = sizeof(WNDCLASSEX);

    wcex.style = CS_HREDRAW | CS_VREDRAW;
    wcex.lpfnWndProc = WndProc;
    wcex.cbClsExtra = 0;
    wcex.cbWndExtra = 0;
    wcex.hInstance = hInstance;
    wcex.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_HUBSWITCHSAMPLE));
    wcex.hCursor = LoadCursor(nullptr, IDC_ARROW);
    wcex.hbrBackground = CreateSolidBrush(RGB(80, 80, 80));
    wcex.lpszMenuName = NULL;
    wcex.lpszClassName = szWindowClass;
    wcex.hIconSm = LoadIcon(wcex.hInstance, MAKEINTRESOURCE(IDI_SMALL));

    return RegisterClassExW(&wcex);
}

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
    hInst = hInstance;

    HWND hWnd = CreateWindowW(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW & ~WS_MAXIMIZEBOX & ~WS_MINIMIZEBOX & ~WS_THICKFRAME,
        CW_USEDEFAULT, 0, 305, 180, nullptr, nullptr, hInstance, nullptr);

    if (!hWnd)
    {
        return FALSE;
    }

    CreateWindowW(L"BUTTON", L"1", WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_OWNERDRAW,
        10, 70, 60, 60, hWnd, (HMENU)1, hInstance, NULL);
    CreateWindowW(L"BUTTON", L"2", WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_OWNERDRAW,
        80, 70, 60, 60, hWnd, (HMENU)2, hInstance, NULL);
    CreateWindowW(L"BUTTON", L"3", WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_OWNERDRAW,
        150, 70, 60, 60, hWnd, (HMENU)3, hInstance, NULL);
    CreateWindowW(L"BUTTON", L"4", WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_OWNERDRAW,
        220, 70, 60, 60, hWnd, (HMENU)4, hInstance, NULL);

    CreateWindowW(L"BUTTON", L"GREEN", WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_OWNERDRAW,
        10, 10, 30, 30, hWnd, (HMENU)5, hInstance, NULL);

    CreateWindowW(L"BUTTON", L"RED", WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_OWNERDRAW,
        250, 10, 30, 30, hWnd, (HMENU)6, hInstance, NULL);

    hEdit = CreateWindowW(L"EDIT", NULL, WS_CHILD | WS_VISIBLE | WS_BORDER | ES_CENTER,
        50, 10, 190, 30, hWnd, NULL, hInstance, NULL);

    // シリアル番号を取得してエディットコントロールに表示
    WCHAR serialNumber[64];
    if (GetSerialNumber(serialNumber, 64))
    {
        SetWindowText(hEdit, serialNumber);
    }
    else
    {
        SetWindowText(hEdit, L"Error: No Device");
    }

    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd);

    FT_STATUS ftStatus;
    DWORD dwDevs;

    if (FT_CreateDeviceInfoList(&dwDevs) != FT_OK)
        return FALSE;
    if (dwDevs == 0)
    {
        MessageBox(hWnd, L"No Device", L"ERR", MB_OK);
        return FALSE;
    }

    ftStatus = FT_Open(0, &ftHandle);
    char cOut = 0x0F;
    DWORD bytesWritten;
    FT_Write(ftHandle, &cOut, 1, &bytesWritten);
    if (ftStatus != FT_OK)
    {
        MessageBox(hWnd, L"USB-HUBV3-VC-4P not found.", L"Error", MB_OK);
        return FALSE;
    }

    return TRUE;
}

BOOL GetSerialNumber(WCHAR* serialNumber, DWORD size)
{
    FT_STATUS ftStatus;
    DWORD numDevs;

    ftStatus = FT_CreateDeviceInfoList(&numDevs);
    if (ftStatus != FT_OK || numDevs == 0)
    {
        return FALSE;
    }

    FT_DEVICE_LIST_INFO_NODE* devInfo = new FT_DEVICE_LIST_INFO_NODE[numDevs];
    ftStatus = FT_GetDeviceInfoList(devInfo, &numDevs);
    if (ftStatus != FT_OK)
    {
        delete[] devInfo;
        return FALSE;
    }

    for (DWORD i = 0; i < numDevs; i++)
    {
        if (devInfo[i].Type == FT_DEVICE_232R)
        {
            MultiByteToWideChar(CP_ACP, 0, devInfo[i].SerialNumber, -1, serialNumber, size);
            delete[] devInfo;
            return TRUE;
        }
    }

    delete[] devInfo;
    return FALSE;
}

BOOL IsValidInput(const std::wstring& input)
{
    std::wregex pattern(L"^[A-Z0-9]{1,20}$");
    return std::regex_match(input, pattern);
}

BOOL WriteToFTDI(const std::wstring& input)
{
    // ここで入力を書き込みます（具体的な方法はFTDIのドキュメントを参照してください）
    // ここでは仮に成功とします
    return TRUE;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    DWORD bytesWritten;
    switch (message)
    {
    case WM_COMMAND:
    {
        int wmId = LOWORD(wParam);

        switch (wmId)
        {
        case 1:
        case 2:
        case 3:
        case 4:
        {
            int index = wmId - 1;
            buttonStates[index].isRed = !buttonStates[index].isRed;
            InvalidateRect((HWND)lParam, NULL, TRUE);
            char cOut = 0;
            cOut =
                (buttonStates[0].isRed ? 0x01 : 0x00) |
                (buttonStates[1].isRed ? 0x02 : 0x00) |
                (buttonStates[2].isRed ? 0x04 : 0x00) |
                (buttonStates[3].isRed ? 0x08 : 0x00);
            FT_Write(ftHandle, &cOut, 1, &bytesWritten);
        }
        break;
        case 6:
        {
            // 予備
        }
        break;
        case 5:
            DestroyWindow(hWnd);
            break;
        case IDM_ABOUT:
            DialogBox(hInst, MAKEINTRESOURCE(IDD_ABOUTBOX), hWnd, About);
            break;
        case IDM_EXIT:
            DestroyWindow(hWnd);
            break;
        default:
            return DefWindowProc(hWnd, message, wParam, lParam);
        }
    }
    break;
    case WM_DRAWITEM:
    {
        LPDRAWITEMSTRUCT pdis = (LPDRAWITEMSTRUCT)lParam;
        if (pdis->CtlType == ODT_BUTTON)
        {
            int index = pdis->CtlID - 1;
            HBRUSH hBrush;

            if (pdis->CtlID == 5)
            {
                hBrush = CreateSolidBrush(RGB(0, 255, 0));
            }
            else if (pdis->CtlID == 6)
            {
                hBrush = CreateSolidBrush(RGB(255, 0, 0));
            }
            else
            {
                ButtonState& state = buttonStates[index];
                hBrush = CreateSolidBrush(state.isRed ? RGB(255, 0, 0) : RGB(10, 10, 10));
            }

            FillRect(pdis->hDC, &pdis->rcItem, hBrush);
            SetBkMode(pdis->hDC, TRANSPARENT);
            SetTextColor(pdis->hDC, (pdis->CtlID == 5 || pdis->CtlID == 6) ? RGB(0, 0, 0) : RGB(255, 255, 255));

            LPCWSTR buttonText;
            switch (pdis->CtlID)
            {
            case 1: buttonText = L"1"; break;
            case 2: buttonText = L"2"; break;
            case 3: buttonText = L"3"; break;
            case 4: buttonText = L"4"; break;
            case 5: buttonText = L""; break;
            case 6: buttonText = L""; break;
            default: buttonText = L""; break;
            }
            DrawText(pdis->hDC, buttonText, -1, &pdis->rcItem, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
            DeleteObject(hBrush);
        }
    }
    break;
    case WM_PAINT:
    {
        PAINTSTRUCT ps;
        HDC hdc = BeginPaint(hWnd, &ps);
        // TODO: HDC を使用する描画コードをここに追加してください...
        EndPaint(hWnd, &ps);
    }
    break;
    case WM_DESTROY:
        PostQuitMessage(0);
        break;
    default:
        return DefWindowProc(hWnd, message, wParam, lParam);
    }
    return 0;
}

// バージョン情報ボックスのメッセージ ハンドラーです。
INT_PTR CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    UNREFERENCED_PARAMETER(lParam);
    switch (message)
    {
    case WM_INITDIALOG:
        return (INT_PTR)TRUE;

    case WM_COMMAND:
        if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)
        {
            EndDialog(hDlg, LOWORD(wParam));
            return (INT_PTR)TRUE;
        }
        break;
    }
    return (INT_PTR)FALSE;
}
